@@ -11,38 +11,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_serialization import jsonutils
-from six.moves.urllib import parse as urllib_parse
+import ujson
 
 from gnocchiclient import utils
 from gnocchiclient.v1 import base
 
 
-def _get_pagination_options(details=False, history=False,
-                            limit=None, marker=None, sorts=None):
-    options = []
-    if details:
-        options.append("details=true")
-    if history:
-        options.append("history=true")
-    if limit:
-        options.append("limit=%d" % limit)
-    if marker:
-        options.append("marker=%s" % urllib_parse.quote(marker))
-    for sort in sorts or []:
-        options.append("sort=%s" % urllib_parse.quote(sort))
-    if options:
-        return "%s" % "&".join(options)
-    else:
-        return ""
-
-
 class ResourceManager(base.Manager):
     url = "v1/resource/"
 
     def list(self, resource_type="generic", details=False, history=False,
              limit=None, marker=None, sorts=None):
-        """List resources
+        """List resources.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -59,12 +39,22 @@ class ResourceManager(base.Manager):
                       ["user_id:desc-nullslast", "project_id:asc"]
         :type sorts: list of str
         """
-        qs = _get_pagination_options(details, history, limit, marker, sorts)
-        url = "%s%s?%s" % (self.url, resource_type, qs)
-        return self._get(url).json()
+        params = utils.build_pagination_options(
+            details, history, limit, marker, sorts)
+        resources = []
+        page_url = "%s%s?%s" % (self.url, resource_type,
+                                utils.dict_to_querystring(params))
+        while page_url:
+            page = self._get(page_url)
+            resources.extend(page.json())
+            if limit is None or len(resources) < limit:
+                page_url = page.links.get("next", {'url': None})['url']
+            else:
+                break
+        return resources
 
     def get(self, resource_type, resource_id, history=False):
-        """Get a resource
+        """Get a resource.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -74,13 +64,12 @@ class ResourceManager(base.Manager):
         :type history: bool
         """
         history = "/history" if history else ""
-        resource_id = utils.encode_resource_id(resource_id)
         url = self.url + "%s/%s%s" % (resource_type, resource_id, history)
         return self._get(url).json()
 
     def history(self, resource_type, resource_id, details=False,
                 limit=None, marker=None, sorts=None):
-        """Get a resource
+        """Get a resource.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -97,13 +86,14 @@ class ResourceManager(base.Manager):
                       ["user_id:desc-nullslast", "project_id:asc"]
         :type sorts: list of str
         """
-        qs = _get_pagination_options(details, False, limit, marker, sorts)
-        resource_id = utils.encode_resource_id(resource_id)
-        url = "%s%s/%s/history?%s" % (self.url, resource_type, resource_id, qs)
+        params = utils.build_pagination_options(details, False, limit, marker,
+                                                sorts)
+        url = "%s%s/%s/history?%s" % (self.url, resource_type, resource_id,
+                                      utils.dict_to_querystring(params))
         return self._get(url).json()
 
     def create(self, resource_type, resource):
-        """Create a resource
+        """Create a resource.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -113,10 +103,10 @@ class ResourceManager(base.Manager):
         return self._post(
             self.url + resource_type,
             headers={'Content-Type': "application/json"},
-            data=jsonutils.dumps(resource)).json()
+            data=ujson.dumps(resource)).json()
 
     def update(self, resource_type, resource_id, resource):
-        """Update a resource
+        """Update a resource.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -125,25 +115,37 @@ class ResourceManager(base.Manager):
         :param resource: Attribute of the resource
         :type resource: dict
         """
-
-        resource_id = utils.encode_resource_id(resource_id)
         return self._patch(
             self.url + resource_type + "/" + resource_id,
             headers={'Content-Type': "application/json"},
-            data=jsonutils.dumps(resource)).json()
+            data=ujson.dumps(resource)).json()
 
     def delete(self, resource_id):
-        """Delete a resource
+        """Delete a resource.
 
         :param resource_id: ID of the resource
         :type resource_id: str
         """
-        resource_id = utils.encode_resource_id(resource_id)
         self._delete(self.url + "generic/" + resource_id)
 
+    def batch_delete(self, query, resource_type="generic"):
+        """Delete a batch of resources based on attribute values.
+
+        :param resource_type: Type of the resource
+        :type resource_type: str
+        """
+        if isinstance(query, dict):
+            return self._delete(
+                self.url + resource_type + "/",
+                headers={'Content-Type': "application/json"},
+                data=ujson.dumps(query)).json()
+        return self._delete(
+            self.url + resource_type + "/?filter=" + query,
+            headers={'Content-Type': "application/json"}).json()
+
     def search(self, resource_type="generic", query=None, details=False,
                history=False, limit=None, marker=None, sorts=None):
-        """List resources
+        """List resources.
 
         :param resource_type: Type of the resource
         :type resource_type: str
@@ -164,16 +166,29 @@ class ResourceManager(base.Manager):
 
         See Gnocchi REST API documentation for the format
         of *query dictionary*
-        http://docs.openstack.org/developer/gnocchi/rest.html#searching-for-resources
+        http://gnocchi.osci.io/rest.html#searching-for-resources
         """
-
         query = query or {}
-        qs = _get_pagination_options(details, history, limit, marker, sorts)
-        url = "v1/search/resource/%s?%s" % (resource_type, qs)
-        return self._post(
-            url, headers={'Content-Type': "application/json"},
-            data=jsonutils.dumps(query)).json()
+        params = utils.build_pagination_options(
+            details, history, limit, marker, sorts)
+        url = "v1/search/resource/%s?%%s" % resource_type
+        resources = []
 
-    def list_types(self):
-        """List the resource types supported by gnocchi"""
-        return self._get(self.url).json()
+        if isinstance(query, dict):
+            page_url = url % utils.dict_to_querystring(params)
+            data = ujson.dumps(query)
+        else:
+            params['filter'] = query
+            page_url = url % utils.dict_to_querystring(params)
+            data = None
+
+        while page_url:
+            page = self._post(
+                page_url, headers={'Content-Type': "application/json"},
+                data=data)
+            resources.extend(page.json())
+            if limit is None or len(resources) < limit:
+                page_url = page.links.get("next", {'url': None})['url']
+            else:
+                break
+        return resources
