@@ -17,7 +17,7 @@ from gnocchiclient import utils
 
 
 class CliStatusShow(show.ShowOne):
-    """Show the status of measurements processing"""
+    """Show the status of measurements processing."""
 
     def take_action(self, parsed_args):
         status = utils.get_client(self).status.get()
