@@ -14,7 +14,9 @@ sudo apt-get update -y && sudo apt-get install -qy \
         python3.9-dev \
         python3.9-distutils \
         python3.11 \
-        python3.11-dev
+        python3.11-dev \
+        python3.12 \
+        python3.12-dev
 
 sudo rm -rf /var/lib/apt/lists/*
 
@@ -22,4 +24,5 @@ export LANG=en_US.UTF-8
 sudo update-locale
 sudo locale-gen $LANG
 
-sudo python3 -m pip install -U pip tox virtualenv
+sudo python3 -m venv /tmp/gnocchi-tox-env
+sudo /tmp/gnocchi-tox-env/bin/python3 -m pip install -U pip tox virtualenv
